/****************************************/
/* v.c:  Mar 22, 1999  <liju99@mit.edu> */
/* viewer of Ju Li's configuration file */
/****************************************/

#include <stdio.h>
#include "W.h"
#define MAX_STRLEN 128
#define TP_STRLEN 2
#define MAX_TP 5
#define GROUP_NAME "atoms"

struct ATOMTYPE
{
    char *name; /* type name */
    double mass; /* atomic mass */
    int occurrence; /* number of atoms of such type */
    int color;
    double radius;  /* interaction radius in A */
} at[MAX_TP+1] =
  {{"Ar", 39.948, 0, W_RED,    1.50},
   {"Si", 28.086, 0, W_GREEN,  1.11},
   {"C",  12.011, 0, W_WHITE,  0.77},
   {"H",  1.0079, 0, W_WHITE,  0.37},
   {"O",  16.000, 0, W_RED,    0.73},
   {"Un", -1.000, 0, W_WHITE,  1.50}};

int main (int argc, char *argv[])
{
    int i, j, k, np, nc[3], *id;
    char buf[MAX_STRLEN], read[MAX_STRLEN],
        *r, *p, **tp;
    double H[3][3], HI[3][3], *mass, *s, *s1;
    FILE *in;
    WSPHERE *sphere; 
    
    if ( argc == 1 )
    { /* no input argument */
	printf ("View config file of JL's format: ");
	for (r=read; r<read+MAX_STRLEN-1; r++)
	    if ((*r=(char)getc(stdin))=='\n') break;
	*r = (char)0;
    }
    else
    {
	strncpy (read, argv[1], MAX_STRLEN-1);
	read[MAX_STRLEN-1] = (char)0;
    }
    for (r=read; (*r==' ')||(*r=='\t'); r++);
    for (p=r; *p!=(char)0; p++)
	if ((*p==' ')||(*p=='\t')) *p=(char)0;
    /* open that configuration file */
    if ( (in=fopen(r,"r")) == NULL )
    {
	printf("v: cannot open config file \"%s\".\n", r);
	exit(1);
    }
    fscanf (in, "Number of particles = %d\n\n", &np);
    s    = (double *) malloc (3*np*sizeof(double));
    s1   = (double *) malloc (3*np*sizeof(double));
    mass = (double *) malloc (np*sizeof(double));
    id   = (int *)    malloc (np*sizeof(int));
    /* name space for atom type */
    tp   = (char **) malloc (np*sizeof(char *));
    tp[0] = (char *) malloc (np*(TP_STRLEN+1)*sizeof(char));
    for (i=1; i<np; i++)
        tp[i] = tp[i-1] + TP_STRLEN + 1;
    
    fscanf (in, "H(1,1) = %lf A\n", &H[0][0]);
    fscanf (in, "H(1,2) = %lf A\n", &H[0][1]);
    fscanf (in, "H(1,3) = %lf A\n", &H[0][2]);
    fscanf (in, "nc(1)  = %d\n\n",  &nc[0]);
    fscanf (in, "H(2,1) = %lf A\n", &H[1][0]);
    fscanf (in, "H(2,2) = %lf A\n", &H[1][1]);
    fscanf (in, "H(2,3) = %lf A\n", &H[1][2]);
    fscanf (in, "nc(2)  = %d\n\n",  &nc[1]);
    fscanf (in, "H(3,1) = %lf A\n", &H[2][0]);
    fscanf (in, "H(3,2) = %lf A\n", &H[2][1]);
    fscanf (in, "H(3,3) = %lf A\n", &H[2][2]);
    fscanf (in, "nc(3)  = %d\n\n",  &nc[2]);
    /* descriptor line */
    fscanf (in, "TP Mass(amu)     sx        sy        sz"
	    "     sx1(1/ns)  sy1(1/ns)  sz1(1/ns)\n");
    for (i=0; i<3*np; i+=3)
    {
	fscanf (in, "%s %lf %lf %lf %lf %lf %lf %lf\n",
		tp[i/3], &mass[i/3], &s[i], &s[i+1], &s[i+2],
		&s1[i],  &s1[i+1], &s1[i+2]);
	for (j=0; j<MAX_TP; j++)
	    if ( (!strcmp(tp[i/3], at[j].name)) &&
		 (mass[i/3]==at[j].mass) )
		break;
	id[i/3] = j;
	/* those that cannot be identified fall */
	/* into the "Unknown" category */
	at[j].occurrence++;
    }
    fclose (in); 
    /* print statistics */	
    fprintf (stdout, "\nIn \"%s\", we found\n", r);
    fprintf (stdout, "-------------------------------------------------\n");
    fprintf (stdout, " Occurrences  Type   Mass   "
	     "Color  Radius[A]\n");
    for (j=0; j<=MAX_TP; j++)
	if (at[j].occurrence>0)
	    fprintf (stdout, "   %7d     %2s   %.3f  %6s  %.3f\n",
		     at[j].occurrence, at[j].name, at[j].mass,
		     Wcolorname[at[j].color-1], at[j].radius);
    fprintf (stdout, "------------------------------------------------\n");
    WLock();
    sphere = (WSPHERE *) WCreateOG(GROUP_NAME, W_SPHERE, np);
    for (i=0; i<np; i++)
    {
	sphere[i].s[0] = s[3*i];
	sphere[i].s[1] = s[3*i+1];
	sphere[i].s[2] = s[3*i+2];
	sphere[i].radius = at[id[i]].radius;
	sphere[i].cid = at[id[i]].color;
    }
    WUnLock();
    WAddWindow
	("W_DEFAULT",  /* $DISPLAY as xhost */
	 r,  /* "win iw" as title */
	 (volatile double(*)[3]) H, /* metric of rs */
	 W_DEFAULT,    /* rs->rw unit */
	 W_DEFAULT_PBC,  /** PBC, no default key **/
	 W_DEFAULT, W_DEFAULT, /* viewport origin xy in rw */
	 W_DEFAULT,    /* viewport origin z in rw */
	 W_DEFAULT, W_DEFAULT, W_DEFAULT, /* x-axes direction */
	 W_DEFAULT, W_DEFAULT, W_DEFAULT, /* y-axes direction */
	 W_DEFAULT, W_DEFAULT, /* screen size contains box */
	 W_DEFAULT,  /* eye position */
	 W_DEFAULT,  /* eye thickness factor */ 
	 W_DEFAULT,  /* view-plate thickness */ 
	 W_DEFAULT,  /* Z-buffer order (and lighting) scheme */
	 W_DEFAULT, W_DEFAULT, /* lighting parameters alpha and beta */ 
	 W_DEFAULT,  /* factor of change */
	 W_DEFAULT_EYE_AID, /* draw the eye_aide or not */
	 W_DEFAULT   /* animation frames/second */);
    sleep(10000000);
    return (0);
} /* end main() */
